{-# LANGUAGE TemplateHaskell #-}

module HAL.Doc.Corpus where

import Control.Lens qualified as L
import Data.Aeson
import Data.Default
import Data.Map.Strict qualified as Map
import GHC.Generics
import HAL.Utils (allLangs, langTitleS, langAbstractS)
import Data.LanguageCodes (ISO639_1(..))
import Protolude
import Servant.API (ToHttpApiData(..))

data Corpus = Corpus
  { _corpus_docid                :: Text
  , _corpus_title                :: [Text]
  , _corpus_title_lang_map       :: Map ISO639_1 [Text]
  , _corpus_abstract             :: [Text]
  , _corpus_abstract_lang_map    :: Map ISO639_1 [Text]
  , _corpus_date                 :: Maybe Text
  , _corpus_source               :: Maybe Text
  , _corpus_authors_names        :: [Text]
  , _corpus_authors_affiliations :: [Text]
  , _corpus_struct_id            :: [Int]
  , _corpus_original             :: Object
  } deriving (Show, Generic)
L.makeLenses ''Corpus

instance Default Corpus where
  def = Corpus "default Id" def def def def def def def def def mempty

instance FromJSON Corpus where
  parseJSON = withObject "Corpus" $ \o -> do
    _corpus_docid                <- o .: "docid"
    _corpus_title                <- o .: "en_title_s" <|> return []
    _corpus_abstract             <- o .: "en_abstract_s" <|> return []
    _corpus_date                 <- o .:? "submittedDate_s"
    _corpus_source               <- o .:? "source_s"
    _corpus_authors_names        <- o .: "authFullName_s" <|> return []
    _corpus_authors_affiliations <- o .: "authOrganism_s" <|> return []
    _corpus_struct_id            <- o .: "structId_i" <|> return []

    titles <-
      mapM (\lang -> do
               mt <- o .:? (langTitleS lang)
               pure $ (\t -> (lang, t)) <$> mt) allLangs
    let _corpus_title_lang_map = Map.fromList $ catMaybes titles

    abstracts <-
      mapM (\lang -> do
               ma <- o .:? (langAbstractS lang)
               pure $ (\a -> (lang, a)) <$> ma) allLangs
    let _corpus_abstract_lang_map = Map.fromList $ catMaybes abstracts

    let _corpus_original = o

    pure $ Corpus { .. }

instance ToHttpApiData Corpus where
  toUrlPiece _ = "docid,en_title_s,fr_title_s,en_abstract_s,fr_abstract_s,submittedDate_s,source_s,authFullName_s,authOrganism_s,structId_i"
